/* Regression webinar Stata 2025/02/11 */
/* Copyright ©2025 by StataCorp LLC */

**# Linear regression
clear all

* load the data
webuse lbw

* cross-sectional
list bwt age race smoke in 1/20, noobs sep(0)

* descriptive statistics
dtable bwt age i.race i.smoke

* OLS estimates
regress bwt age i.race i.smoke
estimates store linear

* show base level permanently
set showbaselevels on, perm

* OLS visualization
preserve
duplicates drop age, force
quietly regress bwt age
local bage = _b[age]
local cons = _b[_cons]

forvalues i = 1/`=_N'{
local y2 = bwt[`i']
local x2 = age[`i']
local x1 = age[`i']
local y1 = age[`i']*`bage' + `cons'
local guts1 "(pcarrowi `y2' `x2' `y1' `x1'" 
local guts2 ", color(stc2) lw(thin) lc(%60) mc(stc2) mlc(%60))"
local guts "`guts' `guts1'`guts2'"
}

graph twoway (lfit bwt age) (scatter bwt age) ///
			 `guts', legend(order(1 "regression line" 2 "actual value")) ///
			 ytitle("Birthweight (grams)") xtitle("Age of mother")
restore				 

* robust standard errors
regress bwt age i.race i.smoke, base vce(robust)
estimates store robust

* bootstrap standard errors
regress bwt age i.race i.smoke, vce(bootstrap, reps(100) seed(123))
estimates store boot

* compare results
etable, estimates(linear robust boot)

* quadratic regression
regress bwt c.age##c.age i.race##i.smoke, base

* test parameters
test 2.race == 3.race
testparm race#smoke

* counterfactual predictions
margins, at(smoke = 0)
margins, at(smoke = 0 race = 1)
margins, at(age=(14(5)45))
marginsplot

* nonlinear combination of parameters
nlcom -_b[age]/(2*_b[age#age])

* visualization of the quadratic relationship with lowest point
local x1 = -_b[age]/(2*_b[age#age])
preserve
margins, at(age = `x1') post
local y1 = _b[_cons]
restore
local x2 = `x1' + 2
local y2 = `y1' - 100
regress bwt c.age##c.age race##smoke, base
estimates store model

quietly margins, at(age=(14(1)45))

marginsplot, noci   ///
   addplot(pcarrowi `y2' `x2' `y1' `x1' (3) "min of bwt at age = 25" || ///
           pci `y1' 20 `y1' 30) ///
   recast(line) legend(off) ///
   title("Quadratic relationship between bwt and age") ///
   ytitle("Prediction of birthweight")

* marginal/partial effect of age
margins, dydx(age)
quietly margins, dydx(age) atmeans

* marginal/partial effect of age across a range of age
margins, dydx(age) at(age=(14(5)45))
marginsplot, yline(0) xline(25.3)

* visualization of the marginal effects
estimates restore model
margins, dydx(age) at(age=(14(5)45)) post
marginsplot, yline(0) xline(25.3)
forvalues i = 1/7{
local me`i': display %6.3f r(table)[1,`i']
}

estimates restore model
quietly margins, at(age=(14(5)45)) post
forvalues i = 1/7{
local pred`i' = r(table)[1,`i']
}

local guts
forvalues i = 14(5)45{
local x1 = `i'

est restore model
quietly margins, at(age = `x1') post
local y1 = _b[_cons]

est restore model
quietly margins, dydx(age) at(age = `i') post
local b = _b[age]

local cons = `y1' - `b'*`x1'

local guts "`guts' (function y = `cons' + `b'*x, range(`=`i'-5' `=`i'+3') lcolor(maroon) lwidth(thin))"

}

local m = 1
forvalues j = 14(5)45{
local guts`m' "(pcarrowi `=`pred`m''+200' `j' `pred`m'' `j' (12) "`me`m''", lcolor(maroon) mcolor(maroon) mlabcolor(maroon))"
local gutsall "`gutsall' `guts`m''"
local ++m
}
macro list _gutsall

est restore model

quietly margins, at(age=(14(1)45))
marginsplot, noci   ///
   addplot(`guts' `gutsall') ///
   recast(line) legend(off) ///  
   title("Quadratic relationship between bwt and age") ///
   ytitle("Prediction of birthweight") plot1opts(lwidth(thick))
   
* interaction counterfactuals and visualization
margins race#smoke

marginsplot, recast(scatter) name(m1, replace)
marginsplot, recastci(rarea) ciopts(fcolor(%20) acolor(%20)) name(m2, replace)
graph combine m1 m2, iscale(0.7)

**# Nonlinear models

* data structure
list bwt low lwt age race smoke in 120/140, noobs sep(0)

* probit model and postestimation
probit low age i.race##c.lwt i.smoke, base

margins, dydx(age)
margins, dydx(age) atmeans

margins smoke, at(age = 25 race = 1)
margins r.smoke, at(age = 25 race = 1)

quietly margins r.race if race == 1 | race == 3, at(lwt=(80(10)250))
marginsplot, yline(0) title("White vs Other")

margins, at(age=generate(age)) at(age=generate(age+4))
margins, at(age=generate(age)) at(age=generate(age+4)) contrast(atcontrast(r))

quietly margins race, at(lwt=(80(10)250)) 
marginsplot, noci

* logistic/logit model
logit low age i.race##c.lwt i.smoke, base
quietly logistic low age i.race##c.lwt i.smoke, base

twoway function normal = normal(x), range(-10 10) || ///
   function logistic = logistic(x), range(-10 10) || ///
   function difference = abs(normal(x) - logistic(x)), lpattern(dash) ///
   lcolor(black) range(-10 10) title("Normal and Logistic CDFs") ytitle("F(x)")

* count outcome
webuse dollhill3, clear
list deaths smoke agecat pyears, noobs sep(0)

poisson deaths smokes i.agecat, exposure(pyears) vce(robust)

margins, predict(pr(5)) 

margins agecat, predict(n)





